#include <stdio.h>
#include <stdlib.h>

int bitratetbl[4][4][16] = { /* version, level, bitrate bits */
    /* V0, L* */
    {
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
    },
    {
        /* V1, L0 */
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
        /* V1, L1 */
        { -1, 32, 64, 96,128,160,192,224,256,288,320,352,384,416,448, -1},
        /* V1, L2 */
        { -1, 32, 48, 56, 64, 80, 96,112,128,160,192,224,256,320,384, -1},
        /* V1, L3 */
        { -1, 32, 40, 48, 56, 64, 80, 96,112,128,160,192,224,256,320, -1}
    },
    {
        /* V2, L0 */
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
        /* V2, L1 */
        { -1, 32, 48, 56, 64, 80, 96,112,128,144,160,176,192,224,256, -1},
        /* V2, L2 */
        { -1,  8, 16, 24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160, -1},
        /* V2, L3 */
        { -1,  8, 16, 24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160, -1}
    },
    {
        /* V2.5, L0 */
        { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
        /* V2.5, L1 */
        { -1, 32, 48, 56, 64, 80, 96,112,128,144,160,176,192,224,256, -1},
        /* V2.5, L2 */
        { -1,  8, 16, 24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160, -1},
        /* V2.5, L3 */
        { -1,  8, 16, 24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160, -1}
    }
};

int freqtbl[4][4] = { /* version, frequency bits */
    {   -1,   -1,   -1,   -1},
    {44100,48000,32000,   -1},
    {22050,24000,16000,   -1},
    {11025,12000, 8000,   -1},
};

int main(int argc, char **argv)
{
    FILE *infile;
    int sync_found = 0;
    int sync_count = 0;
    int samples = 0;

    if (argc!=2) exit(EXIT_FAILURE);

    infile = fopen(argv[1],"rb");
    if (!infile) exit(EXIT_FAILURE);

    while (!feof(infile) && !sync_found) {
        unsigned char b;

        if (fread(&b,1,1,infile)!=1) exit(EXIT_FAILURE);

        if (sync_count == 1 && (b&0xe0) == 0xe0)
            sync_found = 1;
        else if (b==0xff)
            sync_count++;
        else
            sync_count = 0;
    }

    if (!sync_found) exit(EXIT_FAILURE);

    fseek(infile,-2,SEEK_CUR);

    while (!feof(infile)) {
        unsigned char b[4];
        int version = 0;
        int layer = 0;
        int crc = 0;
        int bitrate, frequency;
        int padding = 0;
        off_t morebytes;

        if (fread(b,1,4,infile)!=4) break;
        if (b[0] != 0xff || (b[1]&0xe0) != 0xe0) break;

        switch((b[1]&0x18) >> 3) {
            case 0:
                version = 3; /* 2.5 */
                break;
            case 2:
                version = 2;
                break;
            case 3:
                version = 1;
                break;
            default:
                exit(EXIT_FAILURE);
        }

        switch((b[1]&0x06) >> 1) {
            case 1:
                layer = 3;
                break;
            case 2:
                layer = 2;
                break;
            case 3:
                layer = 1;
                break;
            default:
                exit(EXIT_FAILURE);
        }

        if (!(b[1]&1)) crc = 1;

        bitrate = bitratetbl[version][layer][(b[2]&0xf0)>>4];

        frequency = freqtbl[version][(b[2]&0x0c)>>2];

        if (b[2]&2) padding = 1;

        printf("mpeg%d layer%d bitr=%d freq=%d crc=%d pad=%d\n",version,layer,bitrate,frequency,crc,padding);

        morebytes = (crc?2:0);
        if (layer == 1) {
            samples += 384;
            morebytes += 
                (padding + (12 * bitrate * 1000 / frequency)) * 4;
        } else if (layer == 2 || layer == 3) {
            if (version == 2 && layer == 3) {
                samples += 576;
                morebytes += padding + (72 * bitrate * 1000 / frequency);
            } else {
                samples += 1152;
                morebytes +=
                    padding + (144 * bitrate * 1000 / frequency);
            }
        }

        printf("this frame is %x bytes, next should be at %x\n",morebytes,ftell(infile)+morebytes-4);
        fseek(infile,morebytes-4,SEEK_CUR);
    }

    printf("samples: %d\n",samples);
}
